@@ -11,7 +11,10 @@ use ReturnTypeWillChange;
 use Stringable;
 use Traversable;
 
+use function array_all;
+use function array_any;
 use function array_filter;
+use function array_find;
 use function array_key_exists;
 use function array_keys;
 use function array_map;
@@ -40,18 +43,18 @@ use const ARRAY_FILTER_USE_BOTH;
  * serialize a collection use {@link toArray()} and reconstruct the collection
  * manually.
  *
- * @psalm-template TKey of array-key
- * @psalm-template T
+ * @phpstan-template TKey of array-key
+ * @phpstan-template T
  * @template-implements Collection<TKey,T>
  * @template-implements Selectable<TKey,T>
- * @psalm-consistent-constructor
+ * @phpstan-consistent-constructor
  */
 class ArrayCollection implements Collection, Selectable, Stringable
 {
     /**
      * An array containing the entries of this collection.
      *
-     * @psalm-var array<TKey,T>
+     * @phpstan-var array<TKey,T>
      * @var mixed[]
      */
     private array $elements = [];
@@ -59,8 +62,7 @@ class ArrayCollection implements Collection, Selectable, Stringable
     /**
      * Initializes a new ArrayCollection.
      *
-     * @param array $elements
-     * @psalm-param array<TKey,T> $elements
+     * @phpstan-param array<TKey,T> $elements
      */
     public function __construct(array $elements = [])
     {
@@ -90,13 +92,13 @@ class ArrayCollection implements Collection, Selectable, Stringable
      * instance should be created when constructor semantics have changed.
      *
      * @param array $elements Elements.
-     * @psalm-param array<K,V> $elements
+     * @phpstan-param array<K,V> $elements
      *
      * @return static
-     * @psalm-return static<K,V>
+     * @phpstan-return static<K,V>
      *
-     * @psalm-template K of array-key
-     * @psalm-template V
+     * @phpstan-template K of array-key
+     * @phpstan-template V
      */
     protected function createFrom(array $elements)
     {
@@ -209,6 +211,7 @@ class ArrayCollection implements Collection, Selectable, Stringable
             return;
         }
 
+        /** @phpstan-var TKey $offset */
         $this->set($offset, $value);
     }
 
@@ -246,22 +249,19 @@ class ArrayCollection implements Collection, Selectable, Stringable
      */
     public function exists(Closure $p)
     {
-        foreach ($this->elements as $key => $element) {
-            if ($p($key, $element)) {
-                return true;
-            }
-        }
-
-        return false;
+        return array_any(
+            $this->elements,
+            static fn (mixed $element, mixed $key): bool => (bool) $p($key, $element),
+        );
     }
 
     /**
      * {@inheritDoc}
      *
-     * @psalm-param TMaybeContained $element
+     * @phpstan-param TMaybeContained $element
      *
      * @return int|string|false
-     * @psalm-return (TMaybeContained is T ? TKey|false : false)
+     * @phpstan-return (TMaybeContained is T ? TKey|false : false)
      *
      * @template TMaybeContained
      */
@@ -316,8 +316,6 @@ class ArrayCollection implements Collection, Selectable, Stringable
     /**
      * {@inheritDoc}
      *
-     * @psalm-suppress InvalidPropertyAssignmentValue
-     *
      * This breaks assumptions about the template type, but it would
      * be a backwards-incompatible change to remove this method
      */
@@ -338,7 +336,7 @@ class ArrayCollection implements Collection, Selectable, Stringable
      * {@inheritDoc}
      *
      * @return Traversable<int|string, mixed>
-     * @psalm-return Traversable<TKey, T>
+     * @phpstan-return Traversable<TKey, T>
      */
     #[ReturnTypeWillChange]
     public function getIterator()
@@ -349,12 +347,12 @@ class ArrayCollection implements Collection, Selectable, Stringable
     /**
      * {@inheritDoc}
      *
-     * @psalm-param Closure(T):U $func
+     * @phpstan-param Closure(T):U $func
      *
      * @return static
-     * @psalm-return static<TKey, U>
+     * @phpstan-return static<TKey, U>
      *
-     * @psalm-template U
+     * @phpstan-template U
      */
     public function map(Closure $func)
     {
@@ -372,8 +370,10 @@ class ArrayCollection implements Collection, Selectable, Stringable
     /**
      * {@inheritDoc}
      *
+     * @phpstan-param Closure(T, TKey):bool $p
+     *
      * @return static
-     * @psalm-return static<TKey,T>
+     * @phpstan-return static<TKey,T>
      */
     public function filter(Closure $p)
     {
@@ -385,13 +385,10 @@ class ArrayCollection implements Collection, Selectable, Stringable
      */
     public function findFirst(Closure $p)
     {
-        foreach ($this->elements as $key => $element) {
-            if ($p($key, $element)) {
-                return $element;
-            }
-        }
-
-        return null;
+        return array_find(
+            $this->elements,
+            static fn (mixed $element, mixed $key): bool => (bool) $p($key, $element),
+        );
     }
 
     /**
@@ -399,13 +396,10 @@ class ArrayCollection implements Collection, Selectable, Stringable
      */
     public function forAll(Closure $p)
     {
-        foreach ($this->elements as $key => $element) {
-            if (! $p($key, $element)) {
-                return false;
-            }
-        }
-
-        return true;
+        return array_all(
+            $this->elements,
+            static fn (mixed $element, mixed $key): bool => (bool) $p($key, $element),
+        );
     }
 
     /**
@@ -454,24 +448,26 @@ class ArrayCollection implements Collection, Selectable, Stringable
         return array_slice($this->elements, $offset, $length, true);
     }
 
-    /** @psalm-return Collection<TKey, T>&Selectable<TKey,T> */
+    /** @phpstan-return Collection<TKey, T>&Selectable<TKey,T> */
     public function matching(Criteria $criteria)
     {
+        $accessRawFieldValues = $criteria->isRawFieldValueAccessEnabled();
+
         $expr     = $criteria->getWhereExpression();
         $filtered = $this->elements;
 
         if ($expr) {
-            $visitor  = new ClosureExpressionVisitor();
+            $visitor  = new ClosureExpressionVisitor($accessRawFieldValues);
             $filter   = $visitor->dispatch($expr);
             $filtered = array_filter($filtered, $filter);
         }
 
-        $orderings = $criteria->getOrderings();
+        $orderings = $criteria->orderings();
 
         if ($orderings) {
             $next = null;
             foreach (array_reverse($orderings) as $field => $ordering) {
-                $next = ClosureExpressionVisitor::sortByField($field, $ordering === Criteria::DESC ? -1 : 1, $next);
+                $next = ClosureExpressionVisitor::sortByField($field, $ordering === Order::Descending ? -1 : 1, $next, $accessRawFieldValues);
             }
 
             uasort($filtered, $next);
